# Copyright (c) 2008-2012 testtools developers. See LICENSE for details.

"""Tests for matchers."""

from testtools import (
    Matcher,  # check that Matcher is exposed at the top level for docs.
    TestCase,
)
from testtools.compat import (
    text_repr,
)
from testtools.matchers import (
    Equals,
    MatchesException,
    Raises,
)
from testtools.matchers._impl import (
    Mismatch,
    MismatchDecorator,
    MismatchError,
)
from testtools.tests.helpers import FullStackRunTest

# Silence pyflakes.
Matcher


class TestMismatch(TestCase):
    run_tests_with = FullStackRunTest

    def test_constructor_arguments(self):
        mismatch = Mismatch("some description", {"detail": "things"})
        self.assertEqual("some description", mismatch.describe())
        self.assertEqual({"detail": "things"}, mismatch.get_details())

    def test_constructor_no_arguments(self):
        mismatch = Mismatch()
        self.assertThat(
            mismatch.describe, Raises(MatchesException(NotImplementedError))
        )
        self.assertEqual({}, mismatch.get_details())


class TestMismatchError(TestCase):
    def test_is_assertion_error(self):
        # MismatchError is an AssertionError, so that most of the time, it
        # looks like a test failure, rather than an error.
        def raise_mismatch_error():
            raise MismatchError(2, Equals(3), Equals(3).match(2))

        self.assertRaises(AssertionError, raise_mismatch_error)

    def test_default_description_is_mismatch(self):
        mismatch = Equals(3).match(2)
        e = MismatchError(2, Equals(3), mismatch)
        self.assertEqual(mismatch.describe(), str(e))

    def test_default_description_unicode(self):
        matchee = "\xa7"
        matcher = Equals("a")
        mismatch = matcher.match(matchee)
        e = MismatchError(matchee, matcher, mismatch)
        self.assertEqual(mismatch.describe(), str(e))

    def test_verbose_description(self):
        matchee = 2
        matcher = Equals(3)
        mismatch = matcher.match(2)
        e = MismatchError(matchee, matcher, mismatch, True)
        expected = "Match failed. Matchee: %r\n" "Matcher: %s\n" "Difference: %s\n" % (
            matchee,
            matcher,
            matcher.match(matchee).describe(),
        )
        self.assertEqual(expected, str(e))

    def test_verbose_unicode(self):
        # When assertThat is given matchees or matchers that contain non-ASCII
        # unicode strings, we can still provide a meaningful error.
        matchee = "\xa7"
        matcher = Equals("a")
        mismatch = matcher.match(matchee)
        expected = "Match failed. Matchee: %s\n" "Matcher: %s\n" "Difference: %s\n" % (
            text_repr(matchee),
            matcher,
            mismatch.describe(),
        )
        e = MismatchError(matchee, matcher, mismatch, True)
        self.assertEqual(expected, str(e))


class TestMismatchDecorator(TestCase):
    run_tests_with = FullStackRunTest

    def test_forwards_description(self):
        x = Mismatch("description", {"foo": "bar"})
        decorated = MismatchDecorator(x)
        self.assertEqual(x.describe(), decorated.describe())

    def test_forwards_details(self):
        x = Mismatch("description", {"foo": "bar"})
        decorated = MismatchDecorator(x)
        self.assertEqual(x.get_details(), decorated.get_details())

    def test_repr(self):
        x = Mismatch("description", {"foo": "bar"})
        decorated = MismatchDecorator(x)
        self.assertEqual(
            f"<testtools.matchers.MismatchDecorator({x!r})>", repr(decorated)
        )


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)



# Copyright (c) 2008-2012 testtools developers. See LICENSE for details.

import doctest
import io
import re
import sys

from testtools import TestCase
from testtools.matchers import (
    Annotate,
    Equals,
    LessThan,
    MatchesRegex,
    NotEquals,
)
from testtools.matchers._datastructures import (
    ContainsAll,
    MatchesListwise,
    MatchesStructure,
    MatchesSetwise,
)
from testtools.tests.helpers import FullStackRunTest
from testtools.tests.matchers.helpers import TestMatchersInterface


def run_doctest(obj, name):
    p = doctest.DocTestParser()
    t = p.get_doctest(obj.__doc__, sys.modules[obj.__module__].__dict__, name, "", 0)
    r = doctest.DocTestRunner()
    output = io.StringIO()
    r.run(t, out=output.write)
    return r.failures, output.getvalue()


class TestMatchesListwise(TestCase):
    run_tests_with = FullStackRunTest

    # XXX: Add interface tests.

    def test_docstring(self):
        failure_count, output = run_doctest(MatchesListwise, "MatchesListwise")
        if failure_count:
            self.fail("Doctest failed with %s" % output)


class TestMatchesStructure(TestCase, TestMatchersInterface):
    class SimpleClass:
        def __init__(self, x, y):
            self.x = x
            self.y = y

    matches_matcher = MatchesStructure(x=Equals(1), y=Equals(2))
    matches_matches = [SimpleClass(1, 2)]
    matches_mismatches = [
        SimpleClass(2, 2),
        SimpleClass(1, 1),
        SimpleClass(3, 3),
    ]

    str_examples = [
        ("MatchesStructure(x=Equals(1))", MatchesStructure(x=Equals(1))),
        ("MatchesStructure(y=Equals(2))", MatchesStructure(y=Equals(2))),
        (
            "MatchesStructure(x=Equals(1), y=Equals(2))",
            MatchesStructure(x=Equals(1), y=Equals(2)),
        ),
    ]

    describe_examples = [
        (
            """\
Differences: [
1 != 3: x
]""",
            SimpleClass(1, 2),
            MatchesStructure(x=Equals(3), y=Equals(2)),
        ),
        (
            """\
Differences: [
2 != 3: y
]""",
            SimpleClass(1, 2),
            MatchesStructure(x=Equals(1), y=Equals(3)),
        ),
        (
            """\
Differences: [
1 != 0: x
2 != 0: y
]""",
            SimpleClass(1, 2),
            MatchesStructure(x=Equals(0), y=Equals(0)),
        ),
    ]

    def test_fromExample(self):
        self.assertThat(
            self.SimpleClass(1, 2),
            MatchesStructure.fromExample(self.SimpleClass(1, 3), "x"),
        )

    def test_byEquality(self):
        self.assertThat(self.SimpleClass(1, 2), MatchesStructure.byEquality(x=1))

    def test_withStructure(self):
        self.assertThat(
            self.SimpleClass(1, 2), MatchesStructure.byMatcher(LessThan, x=2)
        )

    def test_update(self):
        self.assertThat(
            self.SimpleClass(1, 2), MatchesStructure(x=NotEquals(1)).update(x=Equals(1))
        )

    def test_update_none(self):
        self.assertThat(
            self.SimpleClass(1, 2),
            MatchesStructure(x=Equals(1), z=NotEquals(42)).update(z=None),
        )


class TestMatchesSetwise(TestCase):
    run_tests_with = FullStackRunTest

    def assertMismatchWithDescriptionMatching(
        self, value, matcher, description_matcher
    ):
        mismatch = matcher.match(value)
        if mismatch is None:
            self.fail(f"{matcher} matched {value}")
        actual_description = mismatch.describe()
        self.assertThat(
            actual_description,
            Annotate(f"{matcher} matching {value}", description_matcher),
        )

    def test_matches(self):
        self.assertIs(None, MatchesSetwise(Equals(1), Equals(2)).match([2, 1]))

    def test_mismatches(self):
        self.assertMismatchWithDescriptionMatching(
            [2, 3],
            MatchesSetwise(Equals(1), Equals(2)),
            MatchesRegex(".*There was 1 mismatch$", re.S),
        )

    def test_too_many_matchers(self):
        self.assertMismatchWithDescriptionMatching(
            [2, 3],
            MatchesSetwise(Equals(1), Equals(2), Equals(3)),
            Equals("There was 1 matcher left over: Equals(1)"),
        )

    def test_too_many_values(self):
        self.assertMismatchWithDescriptionMatching(
            [1, 2, 3],
            MatchesSetwise(Equals(1), Equals(2)),
            Equals("There was 1 value left over: [3]"),
        )

    def test_two_too_many_matchers(self):
        self.assertMismatchWithDescriptionMatching(
            [3],
            MatchesSetwise(Equals(1), Equals(2), Equals(3)),
            MatchesRegex(
                r"There were 2 matchers left over: Equals\([12]\), " r"Equals\([12]\)"
            ),
        )

    def test_two_too_many_values(self):
        self.assertMismatchWithDescriptionMatching(
            [1, 2, 3, 4],
            MatchesSetwise(Equals(1), Equals(2)),
            MatchesRegex(r"There were 2 values left over: \[[34], [34]\]"),
        )

    def test_mismatch_and_too_many_matchers(self):
        self.assertMismatchWithDescriptionMatching(
            [2, 3],
            MatchesSetwise(Equals(0), Equals(1), Equals(2)),
            MatchesRegex(
                r".*There was 1 mismatch and 1 extra matcher: Equals\([01]\)", re.S
            ),
        )

    def test_mismatch_and_too_many_values(self):
        self.assertMismatchWithDescriptionMatching(
            [2, 3, 4],
            MatchesSetwise(Equals(1), Equals(2)),
            MatchesRegex(r".*There was 1 mismatch and 1 extra value: \[[34]\]", re.S),
        )

    def test_mismatch_and_two_too_many_matchers(self):
        self.assertMismatchWithDescriptionMatching(
            [3, 4],
            MatchesSetwise(Equals(0), Equals(1), Equals(2), Equals(3)),
            MatchesRegex(
                ".*There was 1 mismatch and 2 extra matchers: "
                r"Equals\([012]\), Equals\([012]\)",
                re.S,
            ),
        )

    def test_mismatch_and_two_too_many_values(self):
        self.assertMismatchWithDescriptionMatching(
            [2, 3, 4, 5],
            MatchesSetwise(Equals(1), Equals(2)),
            MatchesRegex(
                r".*There was 1 mismatch and 2 extra values: \[[145], [145]\]", re.S
            ),
        )


class TestContainsAllInterface(TestCase, TestMatchersInterface):
    matches_matcher = ContainsAll(["foo", "bar"])
    matches_matches = [["foo", "bar"], ["foo", "z", "bar"], ["bar", "foo"]]
    matches_mismatches = [["f", "g"], ["foo", "baz"], []]

    str_examples = [
        ("MatchesAll(Contains('foo'), Contains('bar'))", ContainsAll(["foo", "bar"])),
    ]

    describe_examples = [
        (
            """Differences: [
'baz' not in 'foo'
]""",
            "foo",
            ContainsAll(["foo", "baz"]),
        )
    ]


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)



# Copyright (c) 2009-2016 testtools developers. See LICENSE for details.

"""Doubles of test result objects, useful for testing unittest code."""

from collections import namedtuple

from testtools.tags import TagContext

__all__ = [
    "Python26TestResult",
    "Python27TestResult",
    "ExtendedTestResult",
    "TwistedTestResult",
    "StreamResult",
]


class LoggingBase:
    """Basic support for logging of results."""

    def __init__(self, event_log=None):
        if event_log is None:
            event_log = []
        self._events = event_log


class Python26TestResult(LoggingBase):
    """A precisely python 2.6 like test result, that logs."""

    def __init__(self, event_log=None):
        super().__init__(event_log=event_log)
        self.shouldStop = False
        self._was_successful = True
        self.testsRun = 0

    def addError(self, test, err):
        self._was_successful = False
        self._events.append(("addError", test, err))

    def addFailure(self, test, err):
        self._was_successful = False
        self._events.append(("addFailure", test, err))

    def addSuccess(self, test):
        self._events.append(("addSuccess", test))

    def startTest(self, test):
        self._events.append(("startTest", test))
        self.testsRun += 1

    def stop(self):
        self.shouldStop = True

    def stopTest(self, test):
        self._events.append(("stopTest", test))

    def wasSuccessful(self):
        return self._was_successful


class Python27TestResult(Python26TestResult):
    """A precisely python 2.7 like test result, that logs."""

    def __init__(self, event_log=None):
        super().__init__(event_log)
        self.failfast = False

    def addError(self, test, err):
        super().addError(test, err)
        if self.failfast:
            self.stop()

    def addFailure(self, test, err):
        super().addFailure(test, err)
        if self.failfast:
            self.stop()

    def addExpectedFailure(self, test, err):
        self._events.append(("addExpectedFailure", test, err))

    def addSkip(self, test, reason):
        self._events.append(("addSkip", test, reason))

    def addUnexpectedSuccess(self, test):
        self._events.append(("addUnexpectedSuccess", test))
        if self.failfast:
            self.stop()

    def startTestRun(self):
        self._events.append(("startTestRun",))

    def stopTestRun(self):
        self._events.append(("stopTestRun",))


class ExtendedTestResult(Python27TestResult):
    """A test result like the proposed extended unittest result API."""

    def __init__(self, event_log=None):
        super().__init__(event_log)
        self._tags = TagContext()

    def addError(self, test, err=None, details=None):
        self._was_successful = False
        self._events.append(("addError", test, err or details))

    def addFailure(self, test, err=None, details=None):
        self._was_successful = False
        self._events.append(("addFailure", test, err or details))

    def addExpectedFailure(self, test, err=None, details=None):
        self._events.append(("addExpectedFailure", test, err or details))

    def addSkip(self, test, reason=None, details=None):
        self._events.append(("addSkip", test, reason or details))

    def addSuccess(self, test, details=None):
        if details:
            self._events.append(("addSuccess", test, details))
        else:
            self._events.append(("addSuccess", test))

    def addUnexpectedSuccess(self, test, details=None):
        self._was_successful = False
        if details is not None:
            self._events.append(("addUnexpectedSuccess", test, details))
        else:
            self._events.append(("addUnexpectedSuccess", test))

    def progress(self, offset, whence):
        self._events.append(("progress", offset, whence))

    def startTestRun(self):
        super().startTestRun()
        self._was_successful = True
        self._tags = TagContext()

    def startTest(self, test):
        super().startTest(test)
        self._tags = TagContext(self._tags)

    def stopTest(self, test):
        self._tags = self._tags.parent
        super().stopTest(test)

    @property
    def current_tags(self):
        return self._tags.get_current_tags()

    def tags(self, new_tags, gone_tags):
        self._tags.change_tags(new_tags, gone_tags)
        self._events.append(("tags", new_tags, gone_tags))

    def time(self, time):
        self._events.append(("time", time))

    def wasSuccessful(self):
        return self._was_successful


class TwistedTestResult(LoggingBase):
    """
    Emulate the relevant bits of :py:class:`twisted.trial.itrial.IReporter`.

    Used to ensure that we can use ``trial`` as a test runner.
    """

    def __init__(self, event_log=None):
        super().__init__(event_log=event_log)
        self._was_successful = True
        self.testsRun = 0

    def startTest(self, test):
        self.testsRun += 1
        self._events.append(("startTest", test))

    def stopTest(self, test):
        self._events.append(("stopTest", test))

    def addSuccess(self, test):
        self._events.append(("addSuccess", test))

    def addError(self, test, error):
        self._was_successful = False
        self._events.append(("addError", test, error))

    def addFailure(self, test, error):
        self._was_successful = False
        self._events.append(("addFailure", test, error))

    def addExpectedFailure(self, test, failure, todo=None):
        self._events.append(("addExpectedFailure", test, failure))

    def addUnexpectedSuccess(self, test, todo=None):
        self._events.append(("addUnexpectedSuccess", test))

    def addSkip(self, test, reason):
        self._events.append(("addSkip", test, reason))

    def wasSuccessful(self):
        return self._was_successful

    def done(self):
        pass


class StreamResult(LoggingBase):
    """A StreamResult implementation for testing.

    All events are logged to _events.
    """

    def startTestRun(self):
        self._events.append(("startTestRun",))

    def stopTestRun(self):
        self._events.append(("stopTestRun",))

    def status(
        self,
        test_id=None,
        test_status=None,
        test_tags=None,
        runnable=True,
        file_name=None,
        file_bytes=None,
        eof=False,
        mime_type=None,
        route_code=None,
        timestamp=None,
    ):
        self._events.append(
            _StatusEvent(
                "status",
                test_id,
                test_status,
                test_tags,
                runnable,
                file_name,
                file_bytes,
                eof,
                mime_type,
                route_code,
                timestamp,
            )
        )


# Convenience for easier access to status fields
_StatusEvent = namedtuple(
    "_Event",
    [
        "name",
        "test_id",
        "test_status",
        "test_tags",
        "runnable",
        "file_name",
        "file_bytes",
        "eof",
        "mime_type",
        "route_code",
        "timestamp",
    ],
)



# Copyright (c) 2008-2010 testtools developers. See LICENSE for details.

"""Utilities for dealing with stuff in unittest.

Legacy - deprecated - use testtools.testsuite.iterate_tests
"""

import warnings

warnings.warn(
    "Please import iterate_tests from testtools.testsuite - "
    "testtools.utils is deprecated.",
    DeprecationWarning,
    stacklevel=2,
)



# Copyright (c) 2008-2012 testtools developers. See LICENSE for details.

import os
import shutil
import tarfile
import tempfile

from testtools import TestCase
from testtools.matchers import (
    Contains,
    DocTestMatches,
    Equals,
)
from testtools.matchers._filesystem import (
    DirContains,
    DirExists,
    FileContains,
    FileExists,
    HasPermissions,
    PathExists,
    SamePath,
    TarballContains,
)


class PathHelpers:
    def mkdtemp(self):
        directory = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, directory)
        return directory

    def create_file(self, filename, contents=""):
        fp = open(filename, "w")
        try:
            fp.write(contents)
        finally:
            fp.close()

    def touch(self, filename):
        return self.create_file(filename)


class TestPathExists(TestCase, PathHelpers):
    def test_exists(self):
        tempdir = self.mkdtemp()
        self.assertThat(tempdir, PathExists())

    def test_not_exists(self):
        doesntexist = os.path.join(self.mkdtemp(), "doesntexist")
        mismatch = PathExists().match(doesntexist)
        self.assertThat("%s does not exist." % doesntexist, Equals(mismatch.describe()))


class TestDirExists(TestCase, PathHelpers):
    def test_exists(self):
        tempdir = self.mkdtemp()
        self.assertThat(tempdir, DirExists())

    def test_not_exists(self):
        doesntexist = os.path.join(self.mkdtemp(), "doesntexist")
        mismatch = DirExists().match(doesntexist)
        self.assertThat(
            PathExists().match(doesntexist).describe(), Equals(mismatch.describe())
        )

    def test_not_a_directory(self):
        filename = os.path.join(self.mkdtemp(), "foo")
        self.touch(filename)
        mismatch = DirExists().match(filename)
        self.assertThat(
            "%s is not a directory." % filename, Equals(mismatch.describe())
        )


class TestFileExists(TestCase, PathHelpers):
    def test_exists(self):
        tempdir = self.mkdtemp()
        filename = os.path.join(tempdir, "filename")
        self.touch(filename)
        self.assertThat(filename, FileExists())

    def test_not_exists(self):
        doesntexist = os.path.join(self.mkdtemp(), "doesntexist")
        mismatch = FileExists().match(doesntexist)
        self.assertThat(
            PathExists().match(doesntexist).describe(), Equals(mismatch.describe())
        )

    def test_not_a_file(self):
        tempdir = self.mkdtemp()
        mismatch = FileExists().match(tempdir)
        self.assertThat("%s is not a file." % tempdir, Equals(mismatch.describe()))


class TestDirContains(TestCase, PathHelpers):
    def test_empty(self):
        tempdir = self.mkdtemp()
        self.assertThat(tempdir, DirContains([]))

    def test_not_exists(self):
        doesntexist = os.path.join(self.mkdtemp(), "doesntexist")
        mismatch = DirContains([]).match(doesntexist)
        self.assertThat(
            PathExists().match(doesntexist).describe(), Equals(mismatch.describe())
        )

    def test_contains_files(self):
        tempdir = self.mkdtemp()
        self.touch(os.path.join(tempdir, "foo"))
        self.touch(os.path.join(tempdir, "bar"))
        self.assertThat(tempdir, DirContains(["bar", "foo"]))

    def test_matcher(self):
        tempdir = self.mkdtemp()
        self.touch(os.path.join(tempdir, "foo"))
        self.touch(os.path.join(tempdir, "bar"))
        self.assertThat(tempdir, DirContains(matcher=Contains("bar")))

    def test_neither_specified(self):
        self.assertRaises(AssertionError, DirContains)

    def test_both_specified(self):
        self.assertRaises(
            AssertionError, DirContains, filenames=[], matcher=Contains("a")
        )

    def test_does_not_contain_files(self):
        tempdir = self.mkdtemp()
        self.touch(os.path.join(tempdir, "foo"))
        mismatch = DirContains(["bar", "foo"]).match(tempdir)
        self.assertThat(
            Equals(["bar", "foo"]).match(["foo"]).describe(),
            Equals(mismatch.describe()),
        )


class TestFileContains(TestCase, PathHelpers):
    def test_not_exists(self):
        doesntexist = os.path.join(self.mkdtemp(), "doesntexist")
        mismatch = FileContains("").match(doesntexist)
        self.assertThat(
            PathExists().match(doesntexist).describe(), Equals(mismatch.describe())
        )

    def test_contains(self):
        tempdir = self.mkdtemp()
        filename = os.path.join(tempdir, "foo")
        self.create_file(filename, "Hello World!")
        self.assertThat(filename, FileContains("Hello World!"))

    def test_matcher(self):
        tempdir = self.mkdtemp()
        filename = os.path.join(tempdir, "foo")
        self.create_file(filename, "Hello World!")
        self.assertThat(filename, FileContains(matcher=DocTestMatches("Hello World!")))

    def test_neither_specified(self):
        self.assertRaises(AssertionError, FileContains)

    def test_both_specified(self):
        self.assertRaises(
            AssertionError, FileContains, contents=[], matcher=Contains("a")
        )

    def test_does_not_contain(self):
        tempdir = self.mkdtemp()
        filename = os.path.join(tempdir, "foo")
        self.create_file(filename, "Goodbye Cruel World!")
        mismatch = FileContains("Hello World!").match(filename)
        self.assertThat(
            Equals("Hello World!").match("Goodbye Cruel World!").describe(),
            Equals(mismatch.describe()),
        )


class TestTarballContains(TestCase, PathHelpers):
    def test_match(self):
        tempdir = self.mkdtemp()

        def in_temp_dir(x):
            return os.path.join(tempdir, x)

        self.touch(in_temp_dir("a"))
        self.touch(in_temp_dir("b"))
        tarball = tarfile.open(in_temp_dir("foo.tar.gz"), "w")
        tarball.add(in_temp_dir("a"), "a")
        tarball.add(in_temp_dir("b"), "b")
        tarball.close()
        self.assertThat(in_temp_dir("foo.tar.gz"), TarballContains(["b", "a"]))

    def test_mismatch(self):
        tempdir = self.mkdtemp()

        def in_temp_dir(x):
            return os.path.join(tempdir, x)

        self.touch(in_temp_dir("a"))
        self.touch(in_temp_dir("b"))
        tarball = tarfile.open(in_temp_dir("foo.tar.gz"), "w")
        tarball.add(in_temp_dir("a"), "a")
        tarball.add(in_temp_dir("b"), "b")
        tarball.close()
        mismatch = TarballContains(["d", "c"]).match(in_temp_dir("foo.tar.gz"))
        self.assertEqual(
            mismatch.describe(), Equals(["c", "d"]).match(["a", "b"]).describe()
        )


class TestSamePath(TestCase, PathHelpers):
    def test_same_string(self):
        self.assertThat("foo", SamePath("foo"))

    def test_relative_and_absolute(self):
        path = "foo"
        abspath = os.path.abspath(path)
        self.assertThat(path, SamePath(abspath))
        self.assertThat(abspath, SamePath(path))

    def test_real_path(self):
        tempdir = self.mkdtemp()
        source = os.path.join(tempdir, "source")
        self.touch(source)
        target = os.path.join(tempdir, "target")
        try:
            os.symlink(source, target)
        except (AttributeError, NotImplementedError):
            self.skipTest("No symlink support")
        self.assertThat(source, SamePath(target))
        self.assertThat(target, SamePath(source))


class TestHasPermissions(TestCase, PathHelpers):
    def test_match(self):
        tempdir = self.mkdtemp()
        filename = os.path.join(tempdir, "filename")
        self.touch(filename)
        permissions = oct(os.stat(filename).st_mode)[-4:]
        self.assertThat(filename, HasPermissions(permissions))


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)



# Copyright (c) 2016 testtools developers. See LICENSE for details.

from testtools import TestCase
from testtools.matchers import Always, Never
from testtools.tests.matchers.helpers import TestMatchersInterface


class TestAlwaysInterface(TestMatchersInterface, TestCase):
    """:py:func:`~testtools.matchers.Always` always matches."""

    matches_matcher = Always()
    matches_matches = [42, object(), "hi mom"]
    matches_mismatches = []

    str_examples = [("Always()", Always())]
    describe_examples = []


class TestNeverInterface(TestMatchersInterface, TestCase):
    """:py:func:`~testtools.matchers.Never` never matches."""

    matches_matcher = Never()
    matches_matches = []
    matches_mismatches = [42, object(), "hi mom"]

    str_examples = [("Never()", Never())]
    describe_examples = [("Inevitable mismatch on 42", 42, Never())]


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)



from testtools import TestCase
from testtools.matchers import (
    Equals,
    NotEquals,
    Not,
)
from testtools.matchers._dict import (
    ContainedByDict,
    ContainsDict,
    KeysEqual,
    MatchesAllDict,
    MatchesDict,
    _SubDictOf,
)
from testtools.tests.matchers.helpers import TestMatchersInterface


class TestMatchesAllDictInterface(TestCase, TestMatchersInterface):
    matches_matcher = MatchesAllDict({"a": NotEquals(1), "b": NotEquals(2)})
    matches_matches = [3, 4]
    matches_mismatches = [1, 2]

    str_examples = [
        ("MatchesAllDict({'a': NotEquals(1), 'b': NotEquals(2)})", matches_matcher)
    ]

    describe_examples = [
        ("""a: 1 == 1""", 1, matches_matcher),
    ]


class TestKeysEqualEmpty(TestCase, TestMatchersInterface):
    matches_matcher = KeysEqual()
    matches_matches = [
        {},
    ]
    matches_mismatches = [
        {"foo": 0, "bar": 1},
        {"foo": 0},
        {"bar": 1},
        {"foo": 0, "bar": 1, "baz": 2},
        {"a": None, "b": None, "c": None},
    ]

    str_examples = [
        ("KeysEqual()", KeysEqual()),
    ]

    describe_examples = [
        ("[] does not match {1: 2}: Keys not equal", {1: 2}, matches_matcher),
    ]


class TestKeysEqualWithList(TestCase, TestMatchersInterface):
    matches_matcher = KeysEqual("foo", "bar")
    matches_matches = [
        {"foo": 0, "bar": 1},
    ]
    matches_mismatches = [
        {},
        {"foo": 0},
        {"bar": 1},
        {"foo": 0, "bar": 1, "baz": 2},
        {"a": None, "b": None, "c": None},
    ]

    str_examples = [
        ("KeysEqual('foo', 'bar')", KeysEqual("foo", "bar")),
    ]

    describe_examples = []

    def test_description(self):
        matchee = {"foo": 0, "bar": 1, "baz": 2}
        mismatch = KeysEqual("foo", "bar").match(matchee)
        description = mismatch.describe()
        self.assertThat(
            description,
            Equals("['bar', 'foo'] does not match %r: Keys not equal" % (matchee,)),
        )


class TestKeysEqualWithDict(TestKeysEqualWithList):
    matches_matcher = KeysEqual({"foo": 3, "bar": 4})


class TestSubDictOf(TestCase, TestMatchersInterface):
    matches_matcher = _SubDictOf({"foo": "bar", "baz": "qux"})

    matches_matches = [
        {"foo": "bar", "baz": "qux"},
        {"foo": "bar"},
    ]

    matches_mismatches = [
        {"foo": "bar", "baz": "qux", "cat": "dog"},
        {"foo": "bar", "cat": "dog"},
    ]

    str_examples = []
    describe_examples = []


class TestMatchesDict(TestCase, TestMatchersInterface):
    matches_matcher = MatchesDict({"foo": Equals("bar"), "baz": Not(Equals("qux"))})

    matches_matches = [
        {"foo": "bar", "baz": None},
        {"foo": "bar", "baz": "quux"},
    ]
    matches_mismatches = [
        {},
        {"foo": "bar", "baz": "qux"},
        {"foo": "bop", "baz": "qux"},
        {"foo": "bar", "baz": "quux", "cat": "dog"},
        {"foo": "bar", "cat": "dog"},
    ]

    str_examples = [
        (
            "MatchesDict({{'baz': {}, 'foo': {}}})".format(
                Not(Equals("qux")), Equals("bar")
            ),
            matches_matcher,
        ),
    ]

    describe_examples = [
        (
            "Missing: {\n"
            "  'baz': Not(Equals('qux')),\n"
            "  'foo': Equals('bar'),\n"
            "}",
            {},
            matches_matcher,
        ),
        (
            "Differences: {\n" "  'baz': 'qux' matches Equals('qux'),\n" "}",
            {"foo": "bar", "baz": "qux"},
            matches_matcher,
        ),
        (
            "Differences: {\n"
            "  'baz': 'qux' matches Equals('qux'),\n"
            "  'foo': 'bop' != 'bar',\n"
            "}",
            {"foo": "bop", "baz": "qux"},
            matches_matcher,
        ),
        (
            "Extra: {\n" "  'cat': 'dog',\n" "}",
            {"foo": "bar", "baz": "quux", "cat": "dog"},
            matches_matcher,
        ),
        (
            "Extra: {\n"
            "  'cat': 'dog',\n"
            "}\n"
            "Missing: {\n"
            "  'baz': Not(Equals('qux')),\n"
            "}",
            {"foo": "bar", "cat": "dog"},
            matches_matcher,
        ),
    ]


class TestContainsDict(TestCase, TestMatchersInterface):
    matches_matcher = ContainsDict({"foo": Equals("bar"), "baz": Not(Equals("qux"))})

    matches_matches = [
        {"foo": "bar", "baz": None},
        {"foo": "bar", "baz": "quux"},
        {"foo": "bar", "baz": "quux", "cat": "dog"},
    ]
    matches_mismatches = [
        {},
        {"foo": "bar", "baz": "qux"},
        {"foo": "bop", "baz": "qux"},
        {"foo": "bar", "cat": "dog"},
        {"foo": "bar"},
    ]

    str_examples = [
        (
            "ContainsDict({{'baz': {}, 'foo': {}}})".format(
                Not(Equals("qux")), Equals("bar")
            ),
            matches_matcher,
        ),
    ]

    describe_examples = [
        (
            "Missing: {\n"
            "  'baz': Not(Equals('qux')),\n"
            "  'foo': Equals('bar'),\n"
            "}",
            {},
            matches_matcher,
        ),
        (
            "Differences: {\n" "  'baz': 'qux' matches Equals('qux'),\n" "}",
            {"foo": "bar", "baz": "qux"},
            matches_matcher,
        ),
        (
            "Differences: {\n"
            "  'baz': 'qux' matches Equals('qux'),\n"
            "  'foo': 'bop' != 'bar',\n"
            "}",
            {"foo": "bop", "baz": "qux"},
            matches_matcher,
        ),
        (
            "Missing: {\n" "  'baz': Not(Equals('qux')),\n" "}",
            {"foo": "bar", "cat": "dog"},
            matches_matcher,
        ),
    ]


class TestContainedByDict(TestCase, TestMatchersInterface):
    matches_matcher = ContainedByDict({"foo": Equals("bar"), "baz": Not(Equals("qux"))})

    matches_matches = [
        {},
        {"foo": "bar"},
        {"foo": "bar", "baz": "quux"},
        {"baz": "quux"},
    ]
    matches_mismatches = [
        {"foo": "bar", "baz": "quux", "cat": "dog"},
        {"foo": "bar", "baz": "qux"},
        {"foo": "bop", "baz": "qux"},
        {"foo": "bar", "cat": "dog"},
    ]

    str_examples = [
        (
            "ContainedByDict({{'baz': {}, 'foo': {}}})".format(
                Not(Equals("qux")), Equals("bar")
            ),
            matches_matcher,
        ),
    ]

    describe_examples = [
        (
            "Differences: {\n" "  'baz': 'qux' matches Equals('qux'),\n" "}",
            {"foo": "bar", "baz": "qux"},
            matches_matcher,
        ),
        (
            "Differences: {\n"
            "  'baz': 'qux' matches Equals('qux'),\n"
            "  'foo': 'bop' != 'bar',\n"
            "}",
            {"foo": "bop", "baz": "qux"},
            matches_matcher,
        ),
        (
            "Extra: {\n" "  'cat': 'dog',\n" "}",
            {"foo": "bar", "cat": "dog"},
            matches_matcher,
        ),
    ]


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)



#!/usr/bin/python

"""Release testtools on Launchpad.

Steps:
 1. Make sure all "Fix committed" bugs are assigned to 'next'
 2. Rename 'next' to the new version
 3. Release the milestone
 4. Upload the tarball
 5. Create a new 'next' milestone
 6. Mark all "Fix committed" bugs in the milestone as "Fix released"

Assumes that NEWS is in the parent directory, that the release sections are
underlined with '~' and the subsections are underlined with '-'.

Assumes that this file is in the 'scripts' directory a testtools tree that has
already had a tarball built and uploaded with 'python setup.py sdist upload
--sign'.
"""

from datetime import datetime, timedelta, tzinfo
import logging
import os
import sys

from launchpadlib.launchpad import Launchpad
from launchpadlib import uris


APP_NAME = "testtools-lp-release"
CACHE_DIR = os.path.expanduser("~/.launchpadlib/cache")
SERVICE_ROOT = uris.LPNET_SERVICE_ROOT

FIX_COMMITTED = "Fix Committed"
FIX_RELEASED = "Fix Released"

# Launchpad file type for a tarball upload.
CODE_RELEASE_TARBALL = "Code Release Tarball"

PROJECT_NAME = "testtools"
NEXT_MILESTONE_NAME = "next"


class _UTC(tzinfo):
    """UTC"""

    def utcoffset(self, dt):
        return timedelta(0)

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return timedelta(0)


UTC = _UTC()


def configure_logging():
    level = logging.INFO
    log = logging.getLogger(APP_NAME)
    log.setLevel(level)
    handler = logging.StreamHandler()
    handler.setLevel(level)
    formatter = logging.Formatter("%(levelname)s: %(message)s")
    handler.setFormatter(formatter)
    log.addHandler(handler)
    return log


LOG = configure_logging()


def get_path(relpath):
    """Get the absolute path for something relative to this file."""
    return os.path.abspath(
        os.path.join(os.path.dirname(os.path.dirname(__file__)), relpath)
    )


def assign_fix_committed_to_next(testtools, next_milestone):
    """Find all 'Fix Committed' and make sure they are in 'next'."""
    fixed_bugs = list(testtools.searchTasks(status=FIX_COMMITTED))
    for task in fixed_bugs:
        LOG.debug(f"{task.title}")
        if task.milestone != next_milestone:
            task.milestone = next_milestone
            LOG.info(f"Re-assigning {task.title}")
            task.lp_save()


def rename_milestone(next_milestone, new_name):
    """Rename 'next_milestone' to 'new_name'."""
    LOG.info(f"Renaming {next_milestone.name} to {new_name}")
    next_milestone.name = new_name
    next_milestone.lp_save()


def get_release_notes_and_changelog(news_path):
    release_notes = []
    changelog = []
    state = None
    last_line = None

    def is_heading_marker(line, marker_char):
        return line and line == marker_char * len(line)

    LOG.debug(f"Loading NEWS from {news_path}")
    with open(news_path) as news:
        for line in news:
            line = line.strip()
            if state is None:
                if is_heading_marker(line, "~") and not last_line.startswith("NEXT"):
                    milestone_name = last_line
                    state = "release-notes"
                else:
                    last_line = line
            elif state == "title":
                # The line after the title is a heading marker line, so we
                # ignore it and change state. That which follows are the
                # release notes.
                state = "release-notes"
            elif state == "release-notes":
                if is_heading_marker(line, "-"):
                    state = "changelog"
                    # Last line in the release notes is actually the first
                    # line of the changelog.
                    changelog = [release_notes.pop(), line]
                else:
                    release_notes.append(line)
            elif state == "changelog":
                if is_heading_marker(line, "~"):
                    # Last line in changelog is actually the first line of the
                    # next section.
                    changelog.pop()
                    break
                else:
                    changelog.append(line)
            else:
                raise ValueError("Couldn't parse NEWS")

    release_notes = "\n".join(release_notes).strip() + "\n"
    changelog = "\n".join(changelog).strip() + "\n"
    return milestone_name, release_notes, changelog


def release_milestone(milestone, release_notes, changelog):
    date_released = datetime.now(tz=UTC)
    LOG.info(f"Releasing milestone: {milestone.name}, date {date_released}")
    release = milestone.createProductRelease(
        date_released=date_released,
        changelog=changelog,
        release_notes=release_notes,
    )
    milestone.is_active = False
    milestone.lp_save()
    return release


def create_milestone(series, name):
    """Create a new milestone in the same series as 'release_milestone'."""
    LOG.info(f"Creating milestone {name} in series {series.name}")
    return series.newMilestone(name=name)


def close_fixed_bugs(milestone):
    tasks = list(milestone.searchTasks())
    for task in tasks:
        LOG.debug(f"Found {task.title}")
        if task.status == FIX_COMMITTED:
            LOG.info(f"Closing {task.title}")
            task.status = FIX_RELEASED
        else:
            LOG.warning(f"Bug not fixed, removing from milestone: {task.title}")
            task.milestone = None
        task.lp_save()


def upload_tarball(release, tarball_path):
    with open(tarball_path) as tarball:
        tarball_content = tarball.read()
    sig_path = tarball_path + ".asc"
    with open(sig_path) as sig:
        sig_content = sig.read()
    tarball_name = os.path.basename(tarball_path)
    LOG.info(f"Uploading tarball: {tarball_path}")
    release.add_file(
        file_type=CODE_RELEASE_TARBALL,
        file_content=tarball_content,
        filename=tarball_name,
        signature_content=sig_content,
        signature_filename=sig_path,
        content_type="application/x-gzip; charset=binary",
    )


def release_project(launchpad, project_name, next_milestone_name):
    testtools = launchpad.projects[project_name]
    next_milestone = testtools.getMilestone(name=next_milestone_name)
    release_name, release_notes, changelog = get_release_notes_and_changelog(
        get_path("NEWS")
    )
    LOG.info(f"Releasing {project_name} {release_name}")
    # Since reversing these operations is hard, and inspecting errors from
    # Launchpad is also difficult, do some looking before leaping.
    errors = []
    tarball_path = get_path(f"dist/{project_name}-{release_name}.tar.gz")
    if not os.path.isfile(tarball_path):
        errors.append(f"{tarball_path} does not exist")
    if not os.path.isfile(tarball_path + ".asc"):
        errors.append("{} does not exist".format(tarball_path + ".asc"))
    if testtools.getMilestone(name=release_name):
        errors.append(f"Milestone {release_name} exists on {project_name}")
    if errors:
        for error in errors:
            LOG.error(error)
        return 1
    assign_fix_committed_to_next(testtools, next_milestone)
    rename_milestone(next_milestone, release_name)
    release = release_milestone(next_milestone, release_notes, changelog)
    upload_tarball(release, tarball_path)
    create_milestone(next_milestone.series_target, next_milestone_name)
    close_fixed_bugs(next_milestone)
    return 0


def main(args):
    launchpad = Launchpad.login_with(
        APP_NAME, SERVICE_ROOT, CACHE_DIR, credentials_file=".lp_creds"
    )
    return release_project(launchpad, PROJECT_NAME, NEXT_MILESTONE_NAME)


if __name__ == "__main__":
    sys.exit(main(sys.argv))



# Copyright (c) 2010, 2016 testtools developers. See LICENSE for details.

__all__ = [
    "NeedsTwistedTestCase",
]

from testtools.helpers import try_import
from testtools import TestCase

defer = try_import("twisted.internet.defer")


class NeedsTwistedTestCase(TestCase):
    def setUp(self):
        super().setUp()
        if defer is None:
            self.skipTest("Need Twisted to run")



# Copyright (c) 2008-2011 testtools developers. See LICENSE for details.

from testtools import TestCase
from testtools.matchers import (
    DocTestMatches,
    Equals,
    LessThan,
    MatchesStructure,
    Mismatch,
    NotEquals,
)
from testtools.matchers._higherorder import (
    AfterPreprocessing,
    AllMatch,
    Annotate,
    AnnotatedMismatch,
    AnyMatch,
    MatchesAny,
    MatchesAll,
    MatchesPredicate,
    MatchesPredicateWithParams,
    Not,
)
from testtools.tests.helpers import FullStackRunTest
from testtools.tests.matchers.helpers import TestMatchersInterface


class TestAllMatch(TestCase, TestMatchersInterface):
    matches_matcher = AllMatch(LessThan(10))
    matches_matches = [
        [9, 9, 9],
        (9, 9),
        iter([9, 9, 9, 9, 9]),
    ]
    matches_mismatches = [
        [11, 9, 9],
        iter([9, 12, 9, 11]),
    ]

    str_examples = [
        ("AllMatch(LessThan(12))", AllMatch(LessThan(12))),
    ]

    describe_examples = [
        (
            "Differences: [\n" "11 >= 10\n" "10 >= 10\n" "]",
            [11, 9, 10],
            AllMatch(LessThan(10)),
        ),
    ]


class TestAnyMatch(TestCase, TestMatchersInterface):
    matches_matcher = AnyMatch(Equals("elephant"))
    matches_matches = [
        ["grass", "cow", "steak", "milk", "elephant"],
        (13, "elephant"),
        ["elephant", "elephant", "elephant"],
        {"hippo", "rhino", "elephant"},
    ]
    matches_mismatches = [
        [],
        ["grass", "cow", "steak", "milk"],
        (13, 12, 10),
        ["element", "hephalump", "pachyderm"],
        {"hippo", "rhino", "diplodocus"},
    ]

    str_examples = [
        ("AnyMatch(Equals('elephant'))", AnyMatch(Equals("elephant"))),
    ]

    describe_examples = [
        (
            "Differences: [\n" "11 != 7\n" "9 != 7\n" "10 != 7\n" "]",
            [11, 9, 10],
            AnyMatch(Equals(7)),
        ),
    ]


class TestAfterPreprocessing(TestCase, TestMatchersInterface):
    def parity(x):
        return x % 2

    matches_matcher = AfterPreprocessing(parity, Equals(1))
    matches_matches = [3, 5]
    matches_mismatches = [2]

    str_examples = [
        (
            "AfterPreprocessing(<function parity>, Equals(1))",
            AfterPreprocessing(parity, Equals(1)),
        ),
    ]

    describe_examples = [
        (
            "0 != 1: after <function parity> on 2",
            2,
            AfterPreprocessing(parity, Equals(1)),
        ),
        ("0 != 1", 2, AfterPreprocessing(parity, Equals(1), annotate=False)),
    ]


class TestMatchersAnyInterface(TestCase, TestMatchersInterface):
    matches_matcher = MatchesAny(DocTestMatches("1"), DocTestMatches("2"))
    matches_matches = ["1", "2"]
    matches_mismatches = ["3"]

    str_examples = [
        (
            "MatchesAny(DocTestMatches('1\\n'), DocTestMatches('2\\n'))",
            MatchesAny(DocTestMatches("1"), DocTestMatches("2")),
        ),
    ]

    describe_examples = [
        (
            """Differences: [
Expected:
    1
Got:
    3

Expected:
    2
Got:
    3

]""",
            "3",
            MatchesAny(DocTestMatches("1"), DocTestMatches("2")),
        )
    ]


class TestMatchesAllInterface(TestCase, TestMatchersInterface):
    matches_matcher = MatchesAll(NotEquals(1), NotEquals(2))
    matches_matches = [3, 4]
    matches_mismatches = [1, 2]

    str_examples = [
        (
            "MatchesAll(NotEquals(1), NotEquals(2))",
            MatchesAll(NotEquals(1), NotEquals(2)),
        )
    ]

    describe_examples = [
        (
            """Differences: [
1 == 1
]""",
            1,
            MatchesAll(NotEquals(1), NotEquals(2)),
        ),
        (
            "1 == 1",
            1,
            MatchesAll(NotEquals(2), NotEquals(1), Equals(3), first_only=True),
        ),
    ]


class TestAnnotate(TestCase, TestMatchersInterface):
    matches_matcher = Annotate("foo", Equals(1))
    matches_matches = [1]
    matches_mismatches = [2]

    str_examples = [("Annotate('foo', Equals(1))", Annotate("foo", Equals(1)))]

    describe_examples = [("2 != 1: foo", 2, Annotate("foo", Equals(1)))]

    def test_if_message_no_message(self):
        # Annotate.if_message returns the given matcher if there is no
        # message.
        matcher = Equals(1)
        not_annotated = Annotate.if_message("", matcher)
        self.assertIs(matcher, not_annotated)

    def test_if_message_given_message(self):
        # Annotate.if_message returns an annotated version of the matcher if a
        # message is provided.
        matcher = Equals(1)
        expected = Annotate("foo", matcher)
        annotated = Annotate.if_message("foo", matcher)
        self.assertThat(
            annotated, MatchesStructure.fromExample(expected, "annotation", "matcher")
        )


class TestAnnotatedMismatch(TestCase):
    run_tests_with = FullStackRunTest

    def test_forwards_details(self):
        x = Mismatch("description", {"foo": "bar"})
        annotated = AnnotatedMismatch("annotation", x)
        self.assertEqual(x.get_details(), annotated.get_details())


class TestNotInterface(TestCase, TestMatchersInterface):
    matches_matcher = Not(Equals(1))
    matches_matches = [2]
    matches_mismatches = [1]

    str_examples = [
        ("Not(Equals(1))", Not(Equals(1))),
        ("Not(Equals('1'))", Not(Equals("1"))),
    ]

    describe_examples = [("1 matches Equals(1)", 1, Not(Equals(1)))]


def is_even(x):
    return x % 2 == 0


class TestMatchesPredicate(TestCase, TestMatchersInterface):
    matches_matcher = MatchesPredicate(is_even, "%s is not even")
    matches_matches = [2, 4, 6, 8]
    matches_mismatches = [3, 5, 7, 9]

    str_examples = [
        (
            "MatchesPredicate({!r}, {!r})".format(is_even, "%s is not even"),
            MatchesPredicate(is_even, "%s is not even"),
        ),
    ]

    describe_examples = [
        ("7 is not even", 7, MatchesPredicate(is_even, "%s is not even")),
    ]


def between(x, low, high):
    return low < x < high


class TestMatchesPredicateWithParams(TestCase, TestMatchersInterface):
    matches_matcher = MatchesPredicateWithParams(
        between, "{0} is not between {1} and {2}"
    )(1, 9)
    matches_matches = [2, 4, 6, 8]
    matches_mismatches = [0, 1, 9, 10]

    str_examples = [
        (
            "MatchesPredicateWithParams({!r}, {!r})({})".format(
                between, "{0} is not between {1} and {2}", "1, 2"
            ),
            MatchesPredicateWithParams(between, "{0} is not between {1} and {2}")(1, 2),
        ),
        (
            "Between(1, 2)",
            MatchesPredicateWithParams(
                between, "{0} is not between {1} and {2}", "Between"
            )(1, 2),
        ),
    ]

    describe_examples = [
        (
            "1 is not between 2 and 3",
            1,
            MatchesPredicateWithParams(between, "{0} is not between {1} and {2}")(2, 3),
        ),
    ]


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)
